@@ -21,6 +21,7 @@ package desktopentry_test
 
 import (
 	"bytes"
+	"fmt"
 	"os"
 	"path/filepath"
 	"strings"
@@ -37,16 +38,17 @@ type desktopentrySuite struct{}
 
 var _ = Suite(&desktopentrySuite{})
 
-const browserDesktopEntry = `
+const browserDesktopEntryBase = `
 [Desktop Entry]
 X-SnapInstanceName=browser
 Version=1.0
 Type=Application
 Name=Web Browser
 X-SnapAppName=browser-app
-Exec=browser %u
+Exec=browser %%u
 Icon = ${SNAP}/default256.png
 Actions=NewWindow;NewPrivateWindow;
+%[1]s
 
 [Something else]
 Name=Not the app name
@@ -65,6 +67,8 @@ Exec=browser -private-window
 Icon=${SNAP}/private.png
 `
 
+var browserDesktopEntry = fmt.Sprintf(browserDesktopEntryBase, "")
+
 func (s *desktopentrySuite) TestParse(c *C) {
 	r := bytes.NewBufferString(browserDesktopEntry)
 	de, err := desktopentry.Parse("/path/browser.desktop", r)
@@ -75,6 +79,34 @@ func (s *desktopentrySuite) TestParse(c *C) {
 	c.Check(de.Exec, Equals, "browser %u")
 	c.Check(de.SnapInstanceName, Equals, "browser")
 	c.Check(de.SnapAppName, Equals, "browser-app")
+	c.Check(de.SnapCommonID, Equals, "")
+	c.Check(de.Actions, HasLen, 2)
+
+	c.Assert(de.Actions["NewWindow"], NotNil)
+	c.Check(de.Actions["NewWindow"].Name, Equals, "Open a New Window")
+	c.Check(de.Actions["NewWindow"].Icon, Equals, "")
+	c.Check(de.Actions["NewWindow"].Exec, Equals, "browser -new-window")
+	c.Check(de.Actions["NewWindow"].SnapAppName, Equals, "browser-app")
+
+	c.Assert(de.Actions["NewPrivateWindow"], NotNil)
+	c.Check(de.Actions["NewPrivateWindow"].Name, Equals, "Open a New Private Window")
+	c.Check(de.Actions["NewPrivateWindow"].Icon, Equals, "${SNAP}/private.png")
+	c.Check(de.Actions["NewPrivateWindow"].Exec, Equals, "browser -private-window")
+	c.Check(de.Actions["NewPrivateWindow"].SnapAppName, Equals, "browser-app")
+}
+
+func (s *desktopentrySuite) TestParseWithCommonID(c *C) {
+	browserDesktopEntryWithCommonID := fmt.Sprintf(browserDesktopEntryBase, "X-SnapCommonID=browser.app")
+	r := bytes.NewBufferString(browserDesktopEntryWithCommonID)
+	de, err := desktopentry.Parse("/path/browser.desktop", r)
+	c.Assert(err, IsNil)
+
+	c.Check(de.Name, Equals, "Web Browser")
+	c.Check(de.Icon, Equals, "${SNAP}/default256.png")
+	c.Check(de.Exec, Equals, "browser %u")
+	c.Check(de.SnapInstanceName, Equals, "browser")
+	c.Check(de.SnapAppName, Equals, "browser-app")
+	c.Check(de.SnapCommonID, Equals, "browser.app")
 	c.Check(de.Actions, HasLen, 2)
 
 	c.Assert(de.Actions["NewWindow"], NotNil)
@@ -461,7 +493,7 @@ Comment[vi]=Truy cập Internet
 Comment[zh_CN]=访问互联网
 Comment[zh_HK]=連線到網際網路
 Comment[zh_TW]=連線到網際網路
-Exec=env BAMF_DESKTOP_FILE_HINT=/var/lib/snapd/desktop/applications/chromium_chromium.desktop /snap/bin/chromium %U
+Exec=/snap/bin/chromium %U
 Terminal=false
 Type=Application
 Icon=/snap/chromium/1193/chromium.png
@@ -509,7 +541,7 @@ Name[uk]=Відкрити нове вікно
 Name[vi]=Mở cửa sổ mới
 Name[zh_CN]=打开新窗口
 Name[zh_TW]=開啟新視窗
-Exec=env BAMF_DESKTOP_FILE_HINT=/var/lib/snapd/desktop/applications/chromium_chromium.desktop /snap/bin/chromium
+Exec=/snap/bin/chromium
 
 [Desktop Action Incognito]
 Name=Open a New Window in incognito mode
@@ -547,7 +579,7 @@ Name[uk]=Відкрити нове вікно у приватному режим
 Name[vi]=Mở cửa sổ mới trong chế độ ẩn danh
 Name[zh_CN]=以隐身模式打开新窗口
 Name[zh_TW]=以匿名模式開啟新視窗
-Exec=env BAMF_DESKTOP_FILE_HINT=/var/lib/snapd/desktop/applications/chromium_chromium.desktop /snap/bin/chromium --incognito
+Exec=/snap/bin/chromium --incognito
 
 [Desktop Action TempProfile]
 Name=Open a New Window with a temporary profile
@@ -585,7 +617,7 @@ Name[ug]=ۋاقىتلىق سەپلىمە ھۆججەت بىلەن يېڭى كۆز
 Name[vi]=Mở cửa sổ mới với hồ sơ tạm
 Name[zh_CN]=以临时配置文件打开新窗口
 Name[zh_TW]=以暫時性個人身分開啟新視窗
-Exec=env BAMF_DESKTOP_FILE_HINT=/var/lib/snapd/desktop/applications/chromium_chromium.desktop /snap/bin/chromium --temp-profile
+Exec=/snap/bin/chromium --temp-profile
 `
 
 func (s *desktopentrySuite) TestParseChromiumDesktopEntry(c *C) {
@@ -595,29 +627,29 @@ func (s *desktopentrySuite) TestParseChromiumDesktopEntry(c *C) {
 
 	c.Check(de.Name, Equals, "Chromium Web Browser")
 	c.Check(de.Icon, Equals, "/snap/chromium/1193/chromium.png")
-	c.Check(de.Exec, Equals, "env BAMF_DESKTOP_FILE_HINT=/var/lib/snapd/desktop/applications/chromium_chromium.desktop /snap/bin/chromium %U")
+	c.Check(de.Exec, Equals, "/snap/bin/chromium %U")
 	c.Check(de.Actions, HasLen, 3)
 
 	c.Assert(de.Actions["NewWindow"], NotNil)
 	c.Check(de.Actions["NewWindow"].Name, Equals, "Open a New Window")
 	c.Check(de.Actions["NewWindow"].Icon, Equals, "")
-	c.Check(de.Actions["NewWindow"].Exec, Equals, "env BAMF_DESKTOP_FILE_HINT=/var/lib/snapd/desktop/applications/chromium_chromium.desktop /snap/bin/chromium")
+	c.Check(de.Actions["NewWindow"].Exec, Equals, "/snap/bin/chromium")
 
 	c.Assert(de.Actions["Incognito"], NotNil)
 	c.Check(de.Actions["Incognito"].Name, Equals, "Open a New Window in incognito mode")
 	c.Check(de.Actions["Incognito"].Icon, Equals, "")
-	c.Check(de.Actions["Incognito"].Exec, Equals, "env BAMF_DESKTOP_FILE_HINT=/var/lib/snapd/desktop/applications/chromium_chromium.desktop /snap/bin/chromium --incognito")
+	c.Check(de.Actions["Incognito"].Exec, Equals, "/snap/bin/chromium --incognito")
 
 	c.Assert(de.Actions["TempProfile"], NotNil)
 	c.Check(de.Actions["TempProfile"].Name, Equals, "Open a New Window with a temporary profile")
 	c.Check(de.Actions["TempProfile"].Icon, Equals, "")
-	c.Check(de.Actions["TempProfile"].Exec, Equals, "env BAMF_DESKTOP_FILE_HINT=/var/lib/snapd/desktop/applications/chromium_chromium.desktop /snap/bin/chromium --temp-profile")
+	c.Check(de.Actions["TempProfile"].Exec, Equals, "/snap/bin/chromium --temp-profile")
 
 	args, err := de.ExpandExec([]string{"http://example.org"})
 	c.Assert(err, IsNil)
-	c.Check(args, DeepEquals, []string{"env", "BAMF_DESKTOP_FILE_HINT=/var/lib/snapd/desktop/applications/chromium_chromium.desktop", "/snap/bin/chromium", "http://example.org"})
+	c.Check(args, DeepEquals, []string{"/snap/bin/chromium", "http://example.org"})
 
 	args, err = de.ExpandActionExec("Incognito", nil)
 	c.Assert(err, IsNil)
-	c.Check(args, DeepEquals, []string{"env", "BAMF_DESKTOP_FILE_HINT=/var/lib/snapd/desktop/applications/chromium_chromium.desktop", "/snap/bin/chromium", "--incognito"})
+	c.Check(args, DeepEquals, []string{"/snap/bin/chromium", "--incognito"})
 }
